import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import {
  FormGroup,
  FormControl,
  Validators,
  AbstractControl,
} from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { BankRatesService } from '../services/bank-rates.service';
import { AgencyService } from '../services/agency-rates.service';
import { ToastrService } from 'ngx-toastr';

interface ValidationRate {
  minRate: number | null;
  maxRate: number | null;
}

interface ValidationResult {
  offerRate?: ValidationRate;
  settlementRate?: ValidationRate;
  RevaluationRate?: ValidationRate;
  BidRate?: ValidationRate;
}

@Component({
  selector: 'app-bank-rates',
  templateUrl: './bank-rates.component.html',
  styleUrls: ['./bank-rates.component.css'],
})
export class BankRatesComponent implements OnInit {
  @ViewChild('manageBankRates') manageBankRatesModal;
  bsModalRef: BsModalRef;
  validationResults: ValidationResult[] = []; // Updated validationResults array

  type: any = 'add';
  isLoading: boolean = false;

  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';

  bankRates: any = [];

  receivingAgents: any = [];
  selectedReceivingAgent: any = 'All';

  receivingCurrencies: any = [];
  agentAccountList: any = [];
  modeofPaymentList: any = [];
  settlementCurrencies: any = [];

  manageBankRatesForm = new FormGroup({
    AutoRateflag: new FormControl(false),
    Active: new FormControl(false),
    Settelment_Rate: new FormControl('', Validators.required),
    Revalution_Rate: new FormControl('', Validators.required),
    Offer_Rate: new FormControl('', Validators.required),
    Bid_Rate: new FormControl('', Validators.required),
    ModeOfPayment_ID: new FormControl('', Validators.required),
    Agent_ID: new FormControl('', Validators.required),
    ToCurrency_Id: new FormControl('', Validators.required),
    ToCountry_ID: new FormControl('None', Validators.required),
    SettlementCurrency: new FormControl('', Validators.required),
    ID: new FormControl(),
    SettlementCCY: new FormControl(''), // Add this line
  });

  constructor(
    private bankService: BankRatesService,
    private modalService: BsModalService,
    private agencyService: AgencyService,
    private toastr: ToastrService
  ) {}

  ngOnInit(): void {
    this.getReceivingCountry();
    this.getAllBankRates();
    this.getSettlementCurrencies();
    this.getReceivingCurrency();
    this.getModeofPaymentList(0);
    this.manageBankRatesForm
      .get('Offer_Rate')
      .setValidators([Validators.required, this.offerRateValidator.bind(this)]);
    this.manageBankRatesForm
      .get('Settelment_Rate')
      .setValidators([
        Validators.required,
        this.settlementRateValidator.bind(this),
      ]);
  }

  offerRateValidator(
    control: AbstractControl
  ): { [key: string]: boolean } | null {
    const bidRate = this.manageBankRatesForm.get('Bid_Rate').value;
    const offerRate = control.value;

    return offerRate >= bidRate ? null : { offerRateInvalid: true };
  }

  settlementRateValidator(
    control: AbstractControl
  ): { [key: string]: boolean } | null {
    const bidRate = this.manageBankRatesForm.get('Bid_Rate').value;
    const settlementRate = control.value;

    return settlementRate >= bidRate ? null : { settlementRateInvalid: true };
  }

  getAllBankRates(countryId: any = 0, agentId: any = 0) {
    this.isLoading = true;
    this.bankService
      .getAllBankRates(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        agentId
      )
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.bankRates = res;
          for (let index = 0; index < this.bankRates.length; index++) {
            const element = this.bankRates[index];
            if (element?.BidRate !== null && element?.BidRate !== undefined) {
              element.BidRate = this.formatWithSixDecimalPlaces(
                element.BidRate
              );
            }
            if (
              element?.OfferRate !== null &&
              element?.OfferRate !== undefined
            ) {
              element.OfferRate = this.formatWithSixDecimalPlaces(
                element.OfferRate
              );
            }
            if (
              element?.RevalutionRate !== null &&
              element?.RevalutionRate !== undefined
            ) {
              element.RevalutionRate = this.formatWithSixDecimalPlaces(
                element.RevalutionRate
              );
            }
            if (
              element?.SettelmentRate !== null &&
              element?.SettelmentRate !== undefined
            ) {
              element.SettelmentRate = this.formatWithSixDecimalPlaces(
                element.SettelmentRate
              );
            }
          }
        },
        error: (err) => {
          this.bankRates = [];
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }

  formatWithSixDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure six decimal places
      return `${integerPart}.${fractionalPart.padEnd(6, '0').slice(0, 6)}`;
    } else {
      // If no decimal places, add six zeros
      return `${integerPart}.000000`;
    }
  }

  getReceivingCountry(userId: any = 0) {
    this.bankService
      .getReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
        },
        error: (err) => {
          this.receivingCountries = [];
        },
        complete: () => {},
      });
  }

  getReceivingAgent(countryId: any = 0) {
    this.bankService
      .getReceivingAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingAgents = res;
        },
        error: (err) => {
          this.receivingAgents = [];
        },
        complete: () => {},
      });
  }

  getReceivingCurrency(countryId: any = 0) {
    this.bankService
      .getReceivingCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCurrencies = res;
        },
        error: (err) => {
          this.receivingCurrencies = [];
        },
        complete: () => {},
      });
  }

  getAgentAccountList(countryId: any = 0, currencyId: any = 0) {
    this.bankService
      .getAgentAccountList(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        currencyId
      )
      .subscribe({
        next: (res: any) => {
          this.agentAccountList = res;
        },
        error: (err) => {
          this.agentAccountList = [];
        },
        complete: () => {},
      });
  }

  getModeofPaymentList(agentId: any = 0) {
    this.bankService
      .getModeofPaymentList(
        getCacheData('admin_token'),
        getCacheData('username'),
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPaymentList = res;
        },
        error: (err) => {
          this.modeofPaymentList = [];
        },
        complete: () => {},
      });
  }

  getSettlementCurrencies() {
    this.agencyService
      .getSettlementCurrencies(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.settlementCurrencies = res;
        },
        error: (err) => {
          this.settlementCurrencies = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openBankRatesModal(type: any) {
    if (type == 'add') {
      this.manageBankRatesForm.reset({
        ToCountry_ID: 'None',
        ToCurrency_Id: 'None',
        Agent_ID: 'None',
        ModeOfPayment_ID: 'None',
      });
    } else {
      this.getReceivingCurrency();
      this.getAgentAccountList();
    }
    this.type = type;
    this.openModal(this.manageBankRatesModal);
  }

  get f() {
    return this.manageBankRatesForm.value;
  }

  addBankRates() {
    if (!this.manageBankRatesForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.ToCountry_ID == 'None') {
      alert('Please Select Receive Country!');
      return;
    }

    if (this.f.ToCurrency_Id == 'None') {
      alert('Please Select Receive Currency!');
      return;
    }

    if (this.f.Agent_ID == 'None') {
      alert('Please Select Agent Account!');
      return;
    }

    if (this.f.ModeOfPayment_ID == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    if (this.f.SettlementCurrency == '') {
      alert('Please Select Settlement Currency!');
      return;
    }

    if (this.f.Offer_Rate < this.f.Bid_Rate) {
      alert(
        'Offer Rate [USD] should be greater than or equal to Bid Rate [USD]!'
      );
      return;
    }

    if (this.f.Settelment_Rate < this.f.Bid_Rate) {
      alert(
        'Settlement Rate [USD] should be greater than or equal to Bid Rate [USD]!'
      );
      return;
    }

    let payload;
    payload = {
      SettlementCurrency: this.f.SettlementCurrency,
      ToCountry_ID: this.f.ToCountry_ID,
      ToCurrency_Id: this.f.ToCurrency_Id,
      Agent_ID: this.f.Agent_ID,
      ModeOfPayment_ID: this.f.ModeOfPayment_ID,
      Bid_Rate: Number(this.f.Bid_Rate),
      Offer_Rate: Number(this.f.Offer_Rate),
      Revalution_Rate: Number(this.f.Revalution_Rate),
      Settelment_Rate: Number(this.f.Settelment_Rate),
      EnteredBy: getCacheData('G_UserID'),
      AutoRateflag: this.f.AutoRateflag,
    };

    if (this.type === 'edit') {
      const formValues = this.manageBankRatesForm.value;
      const { BidRate } = this.validationResults[1];
      const { offerRate } = this.validationResults[2];
      const { settlementRate } = this.validationResults[3];
      const { RevaluationRate } = this.validationResults[4];
      // Validate Bid Rate

      if (BidRate) {
        if (
          Number(formValues.Bid_Rate) <= BidRate.minRate ||
          Number(formValues.Bid_Rate) >= BidRate.maxRate
        ) {
          alert(
            `Bid Rate should be between ${BidRate.minRate} and ${BidRate.maxRate}`
          );
          return;
        }
      }

      // Validate Offer Rate
      if (offerRate) {
        if (
          Number(formValues.Offer_Rate) <= offerRate.minRate ||
          Number(formValues.Offer_Rate) >= offerRate.maxRate
        ) {
          alert(
            `Offer Rate should be between ${offerRate.minRate} and ${offerRate.maxRate}`
          );
          return;
        }
      }

      // Validate Revaluation Rate
      if (RevaluationRate) {
        if (
          Number(formValues.Revalution_Rate) <= RevaluationRate.minRate ||
          Number(formValues.Revalution_Rate) >= RevaluationRate.maxRate
        ) {
          alert(
            `Revaluation Rate should be between ${RevaluationRate.minRate} and ${RevaluationRate.maxRate}`
          );
          return;
        }
      }

      // Validate Settlement Rate
      if (settlementRate) {
        if (
          Number(formValues.Settelment_Rate) <= settlementRate.minRate ||
          Number(formValues.Settelment_Rate) >= settlementRate.maxRate
        ) {
          alert(
            `Settlement Rate should be between ${settlementRate.minRate} and ${settlementRate.maxRate}`
          );
          return;
        }
      }

      payload = [
        {
          ID: this.f.ID,
          SettlementCurrency: this.f.SettlementCurrency,
          ToCountry_ID: this.f.ToCountry_ID,
          ToCurrency_Id: this.f.ToCurrency_Id,
          Agent_ID: this.f.Agent_ID,
          ModeOfPayment_ID: this.f.ModeOfPayment_ID,
          Bid_Rate: Number(this.f.Bid_Rate),
          Offer_Rate: Number(this.f.Offer_Rate),
          Revalution_Rate: Number(this.f.Revalution_Rate),
          Settelment_Rate: Number(this.f.Settelment_Rate),
          EnteredBy: getCacheData('G_UserID'),
          AutoRateflag: this.f.AutoRateflag,
          Active: this.f.Active,
        },
      ];
    }

    const serviceMethod =
      this.type === 'add' ? 'addBankRates' : 'updateBankRates';

    this.bankService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.getAllBankRates();
        this.modalService.hide();
        const message = this.type === 'edit' ? 'updated' : 'added';
        // alert(res);
        this.toastr.success(res);

        this.validationResults = [];
      },
      error: (err) => {
        this.modalService.hide();
        // alert(err?.error?.message || 'Something went wrong');
        this.toastr.error(err?.error?.message);
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  onReceivingCountryChange() {
    if (this.selectedReceivingCountry != 'All') {
      this.getReceivingAgent(this.selectedReceivingCountry);
    }
  }

  closeModal() {
    this.bsModalRef.hide();
    this.validationResults = [];
  }

  editBankRates(bankRates: any) {
    console.log(bankRates);

    let currencyId: any = 0;
    if (bankRates?.CountryID) {
      this.onReceiveCountryChange(bankRates?.CountryID);
    }
    if (this.receivingCurrencies?.length) {
      currencyId = this.receivingCurrencies.find(
        (p) => p.CurrencyShortName == bankRates.ReceiveCurrency
      )?.Currency_Id;
      if (currencyId) {
        this.onReceiveCurrencyChange(bankRates?.CountryID, currencyId);
      }
    }
    if (bankRates?.Agent_ID) {
      this.onAgentAccountChange(bankRates?.Agent_ID);
    }

    this.validateBidRate(1, bankRates?.RatePer, bankRates?.BidRate);
    this.validateOfferRate(2, bankRates?.RatePer, bankRates?.OfferRate);
    this.validateSettlementRate(
      3,
      bankRates?.RatePer,
      bankRates?.SettelmentRate
    );
    this.validateRevaluationRate(
      4,
      bankRates?.RatePer,
      bankRates?.RevalutionRate
    );

    this.manageBankRatesForm.patchValue({
      ID: bankRates?.ID,
      Agent_ID: bankRates?.Agent_ID,
      ToCountry_ID: bankRates?.CountryID,
      ToCurrency_Id: currencyId,
      ModeOfPayment_ID: bankRates?.ModeOfPayment,
      Bid_Rate: bankRates?.BidRate,
      Offer_Rate: bankRates?.OfferRate,
      Revalution_Rate: bankRates?.RevalutionRate,
      Settelment_Rate: bankRates?.SettelmentRate,
      AutoRateflag: bankRates?.AutoRateFlag,
      Active: bankRates?.Active,
      SettlementCurrency: bankRates?.SettlementCCY, // Add this line
    });
    this.openBankRatesModal('edit');
  }

  searchBankRates() {
    let countryId: number | null = 0;
    let agentId: number | null = 0;
    this.validationResults = [];
    if (this.selectedReceivingCountry !== 'All') {
      countryId = this.selectedReceivingCountry;
    }
    if (this.selectedReceivingAgent !== 'All') {
      agentId = this.selectedReceivingAgent;
    }
    this.getAllBankRates(countryId, agentId);
  }

  onReceiveCountryChange(countryId) {
    if (countryId != 'None') {
      this.getReceivingCurrency(countryId);
    }
  }

  onReceiveCurrencyChange(countryId, currencyId) {
    let countryID: any = 0;
    if (this.f.ToCountry_ID != 'None') {
      countryID = this.f.ToCountry_ID;
    } else {
      countryID = countryId;
    }
    if (currencyId != 'None') {
      this.getAgentAccountList(countryID, currencyId);
    }
  }

  onAgentAccountChange(agentId) {
    if (agentId != 'None') {
      this.getModeofPaymentList(agentId);
    }
  }

  isCheckOfferRate(bidRate: number, offerRate: number): boolean {
    return Number(offerRate) < Number(bidRate);
  }

  isCheckSettlementRate(bidRate: number, settlementRate: number): boolean {
    return Number(settlementRate) < Number(bidRate);
  }

  updateBankRates(): void {
    const updateBankRates = this.bankRates
      .filter((con) => con.isChecked)
      .map((con) => {
        const settlementCurrencyId = this.settlementCurrencies.find(
          (c) => c.CurrencyShortName === con.SettlementCCY
        )?.Currency_Id;
        const currencyId = this.receivingCurrencies.find(
          (c) => c.CurrencyShortName === con.ReceiveCurrency
        )?.Currency_Id;
        const modeofPaymentId = this.modeofPaymentList.find(
          (c) => c.ModeOfPayment_Name === con.ModeOfPayment
        )?.ModeOfPayment_ID;

        return {
          ID: con?.ID,
          SettlementCurrency: settlementCurrencyId,
          ToCountry_ID: con?.CountryID,
          ToCurrency_Id: currencyId,
          Agent_ID: con?.Agent_ID,
          ModeOfPayment_ID: modeofPaymentId,
          Bid_Rate: Number(con?.BidRate),
          Offer_Rate: Number(con?.OfferRate),
          Revalution_Rate: Number(con?.RevalutionRate),
          Settelment_Rate: Number(con?.SettelmentRate),
          EnteredBy: getCacheData('G_UserID'),
          AutoRateflag: con?.AutoRateFlag,
          Active: con.Active,
        };
      });
    if (updateBankRates?.length) {
      this.bankService
        .updateBankRates(
          getCacheData('admin_token'),
          getCacheData('username'),
          updateBankRates
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
            this.getAllBankRates();
            // alert(res)
            this.toastr.success(res);
          },
          error: (err) => {
            this.modalService.hide();
            // alert(err?.error?.message || 'Something went wrong');
            this.toastr.error(err?.error?.message);
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  validateOfferRate(index: number, ratePer: any, offerRate: any): void {
    console.log(ratePer, offerRate);
    if (
      !ratePer ||
      offerRate == null ||
      ratePer === undefined ||
      ratePer === ''
    ) {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        offerRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              offerRate: {
                minRate: parseFloat(MinimumRate),
                maxRate: parseFloat(MaximumRate),
              },
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              offerRate: { minRate: null, maxRate: null },
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            offerRate: { minRate: null, maxRate: null },
          };
        },
        complete: () => {},
      });
  }

  validateSettlementRate(
    index: number,
    ratePer: any,
    settlementRate: any
  ): void {
    console.log(ratePer, settlementRate);
    if (
      !ratePer ||
      settlementRate == null ||
      ratePer === undefined ||
      ratePer === ''
    ) {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        settlementRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              settlementRate: {
                minRate: parseFloat(MinimumRate),
                maxRate: parseFloat(MaximumRate),
              },
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              settlementRate: { minRate: null, maxRate: null },
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            settlementRate: { minRate: null, maxRate: null },
          };
        },
        complete: () => {},
      });
  }

  validateRevaluationRate(
    index: number,
    ratePer: any,
    RevaluationRate: any
  ): void {
    if (
      !ratePer ||
      RevaluationRate == null ||
      ratePer === undefined ||
      ratePer === ''
    ) {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        RevaluationRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              RevaluationRate: {
                minRate: parseFloat(MinimumRate),
                maxRate: parseFloat(MaximumRate),
              },
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              RevaluationRate: { minRate: null, maxRate: null },
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            settlementRate: { minRate: null, maxRate: null },
          };
        },
        complete: () => {},
      });
  }

  validateBidRate(index: number, ratePer: any, BidRate: any): void {
    if (
      !ratePer ||
      BidRate == null ||
      ratePer === undefined ||
      ratePer === ''
    ) {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        BidRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              BidRate: {
                minRate: parseFloat(MinimumRate),
                maxRate: parseFloat(MaximumRate),
              },
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              BidRate: { minRate: null, maxRate: null },
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            settlementRate: { minRate: null, maxRate: null },
          };
        },
        complete: () => {},
      });
  }
}
